import requests
import matplotlib.pyplot as plt

# Binance API Endpoint
api_url = 'https://api.binance.com/api/v3/klines'
symbol = 'BTCUSDT'
interval = '1h'  # 1-hour interval

# Request Kline Data
params = {'symbol': symbol, 'interval': interval, 'limit': 1000}  # Adjust limit as needed
response = requests.get(api_url, params=params)
kline_data = response.json()

# Extracting timestamp, open, high, low, close prices
timestamps = [entry[0] for entry in kline_data]
opens = [float(entry[1]) for entry in kline_data]
highs = [float(entry[2]) for entry in kline_data]
lows = [float(entry[3]) for entry in kline_data]
closes = [float(entry[4]) for entry in kline_data]

# Plotting Candlestick Chart
fig, ax = plt.subplots()
ax.plot(timestamps, closes, label='Closing Prices', color='blue')

# Calculating and Plotting Support Line
support_line = min(lows)
ax.axhline(y=support_line, color='green', linestyle='--', label='Support Line')

# Calculating and Plotting Resistance Line
resistance_line = max(highs)
ax.axhline(y=resistance_line, color='red', linestyle='--', label='Resistance Line')

# Calculating and Plotting Trend Line
# Using two lowest points to determine trend line
lowest_points_indices = sorted(range(len(lows)), key=lambda k: lows[k])[:2]
trend_line = sum([lows[i] for i in lowest_points_indices]) / 2
ax.axhline(y=trend_line, color='purple', linestyle='--', label='Trend Line')

# Customize the plot
ax.set_title(f'{symbol} Price Analysis')
ax.set_xlabel('Timestamp')
ax.set_ylabel('Price (USDT)')
ax.legend()

# Show the plot
plt.show()

