import numpy as np
from sklearn.linear_model import LinearRegression
import matplotlib.pyplot as plt
from scipy.signal import argrelextrema

import requests
import json

url = 'http://192.168.99.1:8080/sma'
candlesResp = requests.get(url)
testSMA = json.loads(candlesResp.text)

# 准备数据
y = np.array(testSMA)
x = np.arange(1, y.size + 1)

minima_indices = argrelextrema(y, np.less)[0]
maxima_indices = argrelextrema(y, np.greater)[0]
print(minima_indices)
print(maxima_indices)

indices = np.sort(np.concatenate((minima_indices, maxima_indices)))

#indices 头部添加第一个元素0
indices = np.insert(indices, 0, 0)

#indices 尾部添加最后一个元素y.size
indices = np.append(indices, y.size)

# 绘制原始数据
plt.scatter(x, y, label='Original Data')

# 绘制相对极小值点和相对极大值点
plt.scatter(x[minima_indices], y[minima_indices], c='r', marker='o', label='Minima')
plt.scatter(x[maxima_indices], y[maxima_indices], c='g', marker='o', label='Maxima')

# 使用 zip 函数生成相邻的元素对
pairs = zip(indices, indices[1:])

# 迭代处理相邻元素
for current, next_item in pairs:
    print(f"Current: {current}, Next: {next_item}")

    x2,y2 = x[current:next_item+1], y[current:next_item+1]
    X2 = x2.reshape(-1, 1)
    model2 = LinearRegression()
    model2.fit(X2, y2)
    plt.plot(x2, model2.predict(X2), color='red', label='Linear Fit '+str(current) )

plt.legend()
plt.show()