#!/usr/bin/env python3
import requests
import pandas as pd
import matplotlib.pyplot as plt

# 替换成您的币安API密钥
api_key = 'YOUR_API_KEY'
api_secret = 'YOUR_API_SECRET'

# 获取K线数据的函数
def get_binance_klines(symbol, interval, limit=500):
    base_url = 'https://api.binance.com/api/v1/klines'
    params = {
        'symbol': symbol,
        'interval': interval,
        'limit': limit
    }
    response = requests.get(base_url, params=params)
    
    if response.status_code == 200:
        klines = response.json()
        df = pd.DataFrame(klines, columns=['timestamp', 'open', 'high', 'low', 'close', 'volume', 'close_time', 'quote_asset_volume', 'number_of_trades', 'taker_buy_base_asset_volume', 'taker_buy_quote_asset_volume', 'ignore'])
        df['timestamp'] = pd.to_datetime(df['timestamp'], unit='ms')
        df.set_index('timestamp', inplace=True)
        df[['open', 'high', 'low', 'close', 'volume']] = df[['open', 'high', 'low', 'close', 'volume']].astype(float)
        return df
    else:
        print(f"Error fetching data. Status code: {response.status_code}")
        return None

# 计算Volume Profile的函数
def calculate_volume_profile(data, bins=20):
    data['VolumeProfile'] = pd.cut(data['close'], bins=bins, labels=False)  # 修改这里的列名
    volume_profile = data.groupby('VolumeProfile')['volume'].sum()  # 修改这里的列名
    return volume_profile

# 使用币安的BTCUSDT交易对，时间间隔为1小时，获取最近500个K线数据
symbol = 'BTCUSDT'
interval = '1h'
klines_df = get_binance_klines(symbol, interval, limit=500)

if klines_df is not None:
    # 计算Volume Profile
    volume_profile = calculate_volume_profile(klines_df, bins=20)
    
    # 绘制Volume Profile图表
    plt.bar(volume_profile.index, volume_profile.values, width=1, alpha=0.7, color='blue')
    plt.title('Volume Profile')
    plt.xlabel('Price Range')
    plt.ylabel('Volume')
    plt.show()
