#include <iostream>
#include <vector>
#include <cmath>
#include <numeric>
#include <cassert>

// Function to find turning points (peaks and troughs)
// Peaks are represented as negative indices and troughs as positive indices
#if 1
std::vector<size_t> findTurningPoints(const std::vector<double>& sma, size_t range, double threshold = 0.001) {
    std::vector<size_t> turningPoints;
    auto n = sma.size();

    bool increasing = true, decreasing = true; // for monotonicity iteration range same as (peak/trough range)

    for (size_t i = range; i < n - range; ++i) {
        bool isPeak = true, isTrough = true;
        for (size_t j = 1; j <= range; ++j) {
            if (sma[i] <= sma[i - j] || sma[i] <= sma[i + j]) {
                isPeak = false;
            }
            if (sma[i] >= sma[i - j] || sma[i] >= sma[i + j]) {
                isTrough = false;
            }
        }

        if (isPeak) {
            turningPoints.push_back(i);
        } else if (isTrough) {
            turningPoints.push_back(i);
        }

        if (sma[i] > sma[i - 1]) decreasing = false; // for strict monotonicity check
        if (sma[i] < sma[i - 1]) increasing = false;
    }

    if (increasing || decreasing) {
        assert(turningPoints.empty()); // must be
        std::cout << (increasing ? "increasing" : "decreasing") << " array then assgin pos 0." << std::endl;
        turningPoints.push_back(0);
    }

    return turningPoints;
}
#else
std::vector<size_t> findTurningPoints(const std::vector<double>& sma, size_t range) {
    std::vector<size_t> turningPoints;
    auto n = sma.size();

    bool increasing = true, decreasing = true; // for monotonicity iteration range same as (peak/trough range)

    for (size_t i = range; i < n - range; ++i) {
        bool isPeak = true, isTrough = true;
        for (size_t j = 1; j <= range; ++j) {
            if (sma[i] <= sma[i - j] || sma[i] <= sma[i + j]) {
                isPeak = false;
            }
            if (sma[i] >= sma[i - j] || sma[i] >= sma[i + j]) {
                isTrough = false;
            }
        }
        if (isPeak) {
            turningPoints.push_back(i);
        } else if (isTrough) {
            turningPoints.push_back(i);
        }

        if (sma[i] > sma[i - 1]) decreasing = false; // for monotonicity check
        if (sma[i] < sma[i - 1]) increasing = false;
    }

    if (increasing || decreasing) {
        assert(turningPoints.empty()); // must be
        std::cout << (increasing ? "increasing" : "decreasing") << " array then assgin pos 0." << std::endl;
        turningPoints.push_back(0);
    }

    return turningPoints;
}
#endif

int main() {
    std::vector<double> sma = {
//68322.5, 68328.3, 68323.7, 68319.2, 68314.6, 68310, 68305.4, 68300.8, 68296.2, 68291.6, 68293.9, 68296.2, 68298.4, 68300.7, 68303, 68305.3, 68307.5, 68309.8, 68318.2, 68326.7, 68335.1, 68343.6, 68352, 68360.4, 68368.9, 68377.3, 68387.5, 68397.7, 68407.9, 68418.1, 68428.3, 68438.5, 68448.7, 68458.8, 68468, 68477.2, 68486.3, 68495.5, 68504.7, 68513.8, 68523, 68532.2, 68543.2, 68554.2, 68565.2, 68576.2, 68587.2, 68598.2, 68609.2, 68620.2, 68630.9, 68641.6, 68652.2, 68662.9, 68673.6, 68684.2, 68694.9, 68705.6, 68713.1, 68720.7, 68728.2, 68735.8, 68743.3, 68750.9, 68758.4, 68766, 68773.4, 68780.9, 68788.3, 68795.8, 68803.3, 68810.7, 68818.2, 68825.6, 68834.8, 68844, 68853.1, 68862.3, 68871.5, 68880.6, 68889.8, 68899, 68908.2, 68917.5, 68926.8, 68936, 68945.3, 68954.6, 68963.8, 68973.1, 68978.7, 68984.2, 68989.8, 68995.4, 69000.9, 69006.5, 69012.1, 69017.6, 69024, 69030.3, 69036.7, 69043, 69049.4, 69055.7, 69062.1, 69068.4, 69071.9, 69075.3, 69078.7, 69082.1, 69085.6, 69089, 69092.4, 69095.8, 69098.3, 69100.7, 69103.1, 69105.5, 69108, 69110.4, 69112.8, 69115.2, 69137.8, 69160.4, 69182.9, 69205.5, 69228, 69250.6, 69273.1, 69295.7, 69311.4, 69327.1, 69342.8, 69358.5, 69374.2, 69389.9, 69405.6, 69421.2, 69440.8, 69460.4, 69480, 69499.6, 69519.2, 69538.8, 69558.4, 69577.9, 69595.8, 69613.6, 69631.4, 69649.3, 69667.1, 69684.9, 69702.8, 69720.6, 69742.8, 69765, 69787.2, 69809.4, 69831.6, 69853.7, 69875.9, 69898.1, 69918.5, 69938.8, 69959.2, 69979.5, 69999.9, 70020.2, 70040.6, 70060.9, 70080.9, 70100.9, 70120.9, 70140.9, 70161, 70181, 70201, 70221, 70244.1, 70267.3, 70290.4
//64298, 64297, 64396, 64295.1, 64294.1, 64292.8, 64291.5, 64290.2, 64288.9, 64287.6, 64286.2, 64284.9, 64283.6, 64274.5, 64265.3, 64256.2, 64247.1, 64237.9, 64228.8, 64219.6, 64210.5, 64201.4
//100, 98, 98, 97 ,96, 95, 94, 93, 92, 91, 90, 89, 88, 86, 85, 84, 82, 81, 80, 77, 76, 68, 64, 62
//100, 98, 98, 97 ,96, 95, 94, 93, 92, 91, 90, 90, 188, 188, 185, 184, 182, 181, 180, 177, 176, 175, 174, 172
//69050.781749999864, 69055.222499999858, 69059.663249999852, 69064.103999999847, 69056.993666666516, 69049.883333333171,
//  69042.772999999841, 69035.662666666511, 69028.552333333166, 69021.441999999835, 69014.331666666505, 69007.22133333316, 69003.409916666496, 68999.598499999833,
//  68995.787083333169, 68991.975666666491, 68988.164249999827, 68984.352833333163, 68980.5414166665, 68976.729999999821, 68969.949416666495, 68963.168833333155,
//  68956.388249999829, 68949.607666666489, 68942.827083333163, 68936.046499999837, 68929.265916666496, 68922.48533333317, 68912.656749999835, 68902.828166666499,
//  68892.999583333178, 68883.170999999842, 68873.342416666506, 68863.513833333185, 68853.685249999849, 68843.856666666514, 68839.826999999859, 68835.79733333319,
//  68831.767666666521, 68827.737999999867, 68823.708333333198, 68819.678666666543, 68815.648999999874, 68811.619333333205, 68809.506916666549, 68807.394499999878,
//  68805.282083333208, 68803.169666666538, 68801.057249999867, 68798.944833333197, 68796.832416666526, 68794.71999999987, 68792.475999999864, 68790.231999999858,
//  68787.987999999852, 68785.743999999861, 68783.499999999854, 68781.255999999848, 68779.011999999842, 68776.767999999851, 68776.266499999852, 68775.764999999854,
//  68775.263499999855, 68774.761999999857, 68774.260499999858, 68773.75899999986, 68773.257499999861, 68772.755999999863, 68772.672916666532, 68772.589833333201,
//  68772.506749999869, 68772.423666666538, 68772.340583333207, 68772.257499999876, 68772.174416666545, 68772.091333333214, 68755.844999999885, 68739.598666666556,
//  68723.352333333227, 68707.105999999898, 68690.859666666569, 68674.61333333324, 68658.366999999911, 68642.120666666582, 68625.658499999918, 68609.196333333239,
//  68592.734166666574, 68576.27199999991, 68559.809833333245, 68543.347666666581, 68526.885499999917, 68510.423333333252, 68503.24316666658, 68496.062999999907,
//  68488.882833333249, 68481.702666666577, 68474.522499999905, 68467.342333333247, 68460.162166666574, 68452.981999999902, 68449.393749999901, 68445.805499999915,
//  68442.217249999914, 68438.628999999914, 68435.040749999913, 68431.452499999912, 68427.864249999911, 68424.275999999911, 68419.217583333244, 68414.159166666592,
//  68409.100749999925, 68404.042333333258, 68398.983916666592, 68393.925499999925, 68388.867083333258, 68383.808666666591, 68371.395916666603, 68358.983166666614,
//  68346.570416666611, 68334.157666666622, 68321.744916666619, 68309.332166666631, 68296.919416666642, 68284.506666666639, 68283.644999999975, 68282.783333333311,
//  68281.921666666662, 68281.059999999998, 68280.198333333334, 68279.33666666667, 68278.475000000006, 68277.613333333342, 68275.224083333349, 68272.834833333342,
  68270.445583333349,
  68268.056333333341,
  68265.667083333348,
  68263.277833333341,
  68260.888583333348,
#if 1
  68258.499333333355, //
  68259.07925000001,
  68259.659166666679,
  68260.239083333348,
  68260.819000000003,
  68261.398916666672,
  68261.978833333342,
  68262.558750000011, 
  68263.138666666666, 
  68260.514750000002, //
#endif
  68257.890833333338, 
  68255.266916666675, 
  68252.642999999996, 
  68250.019083333333, 
  68247.395166666669, 
  68244.771250000005, 
  68242.147333333327, 
  68233.724499999997, 
  68225.301666666666, 
  68216.878833333321, 
  68208.455999999991, 
  68200.033166666661, 
  68191.610333333316, 
  68183.187499999985, 
  68174.764666666655, 
  68165.405583333311, 
  68156.046499999982, 
  68146.687416666653, 
  68137.328333333324, 
  68127.969249999995, 
  68118.610166666651, 
  68109.251083333322, 
  68099.891999999993, 
  68090.664499999999, 
  68081.436999999991, 
  68072.209499999997, 
  68062.982000000004, 
  68053.754499999995, 
  68044.527000000002, 
  68035.299500000008, 
  68026.072,
  68020.058166666669
    };

    std::vector<size_t> turningPoints = findTurningPoints(sma, 10);//前10个元素和最后10个元素不会进入判断区间;其次，单调性的数组也不会产生拐点

    if (turningPoints.empty()) {
        std::cout << "No turning points found." << std::endl;
        return 1;
    }

    // Determine the split point (first turning point)
	for (auto each : turningPoints) { std::cout << each << ", "; }
    std::cout  << sma.at(turningPoints.back()) << std::endl;

	return 0;

}
