#include <iostream>
#include <vector>
#include <cmath>

// Function to find turning points (peaks and troughs)
std::pair<std::vector<int>, std::vector<int>> findTurningPoints(const std::vector<double>& sma, double threshold) {
    std::vector<int> peaks;
    std::vector<int> troughs;

    for (size_t i = 1; i < sma.size() - 1; ++i) {
        if (std::abs(sma[i] - sma[i - 1]) > threshold || std::abs(sma[i] - sma[i + 1]) > threshold) {
            if (sma[i] > sma[i - 1] && sma[i] > sma[i + 1]) {
                peaks.push_back(i);
            } else if (sma[i] < sma[i - 1] && sma[i] < sma[i + 1]) {
                troughs.push_back(i);
            }
        }
    }

    return {peaks, troughs};
}

int main() {
    std::vector<double> sma = {100, 103, 109, 110, 109, 110, 109, 110, 113, 110, 109, 108, 107, 106, 102, 99};
    double threshold = 2.0; // Example threshold value

    auto [peaks, troughs] = findTurningPoints(sma, threshold);

    std::cout << "Peaks at indices: ";
    for (int idx : peaks) {
        std::cout << idx << " ";
    }
    std::cout << "\n";

    std::cout << "Troughs at indices: ";
    for (int idx : troughs) {
        std::cout << idx << " ";
    }
    std::cout << "\n";

    return 0;
}

