#include <type_traits>
#include <iostream>

// 假设 EMA、RSI、SMA、SAR、MACD 是一些特定的类型
struct EMA {};
struct RSI {};
struct SMA {};
struct SAR {};
struct MACD {};

// 类型分类，用于判断类型是否是 SAR 或 MACD
template <typename T>
struct is_sar_or_macd : std::false_type {};

template <>
struct is_sar_or_macd<SAR> : std::true_type {};

template <>
struct is_sar_or_macd<MACD> : std::true_type {};

// 主模板类定义，适用于 EMA, RSI, SMA
template <typename CalcType, typename = void>
class Talib {
public:
    void calculate() {
        std::cout << "Generic Talib calculation for EMA, RSI, SMA\n";
    }
};

// 偏特化模板，适用于 SAR 和 MACD
template <typename CalcType>
class Talib<CalcType, std::enable_if_t<is_sar_or_macd<CalcType>::value>> {
public:
    void calculate() {
        std::cout << "Specialized Talib calculation for SAR and MACD\n";
    }
};

int main() {
    Talib<EMA> talibEma;
    talibEma.calculate();  // 输出: Generic Talib calculation for EMA, RSI, SMA

    Talib<SAR> talibSar;
    talibSar.calculate();  // 输出: Specialized Talib calculation for SAR and MACD

    Talib<MACD> talibMacd;
    talibMacd.calculate();  // 输出: Specialized Talib calculation for SAR and MACD

    return 0;
}

